//DJS 5_6_16

`timescale 1ps/1ps
module pulse_generator
(
	clk,
	sync,
	csb,
	cnvclk,
	sck_gate,
	rx_reset,
	num_bits
);

input clk,sync,num_bits;
output csb,cnvclk,sck_gate,rx_reset;

reg csb,flag,cnvclk,sck_gate,rx_reset;
reg [7:0] sck_length;
reg [7:0] tsckgateh;									

wire [5:0] num_bits;

parameter [6:0] tcsbh = 7'd0;  					// Time for 'csb' to go high; 2 clock cycles after 'sync' rising edge
parameter [6:0] csb_length = 7'd6;				// Yields 'CNVL' pulsewidth of 35nsec
parameter [6:0] tcnvclkh = 7'd2;  				// Time for 'cnvclk' to go high; 4 clock cycles after 'sync' rising edge
parameter [6:0] cnvclk_length = 7'd4;			// Yields 'CNVCLK' pulsewidth of 25nsec
parameter [6:0] trx_reseth = 7'd6;  			// Time for 'rx_reset' to go low; 8 clock cycles after 'sync' rising edge
parameter [6:0] rx_reset_length = 7'd3;	   // Yields 'rx_reset' pulsewidth of 13nsec

// Local wires and registers
reg [6:0] count;						// master counter
//this register resets to '0' while the 'sync' pulse from the PLL is 'high'.
//the following pulses are timed by this count: 'csb', 'sck_gate', 'rx_reset', 'cnvclk_gate'

initial
begin
  flag = 0;
  count = 0;
  tsckgateh = 7'd8;			// Time for 'sck_gate' to go high; 10 clock cycles after 'sync' rising edge
									// Yields first 'SCK' edge 12nsec after 'CSB' falls
end

always @ (clk)  //decode the number of bits
begin
case(num_bits)
	5'd16:
		sck_length = 7'd32;
	5'd14:
		sck_length = 7'd32;
	5'd12:
		sck_length = 7'd28;
	default:
		sck_length = 7'd32;
endcase
end

always @ (posedge clk)
begin
	if(sync)
	begin
		if(flag == 0)
		begin
			count <= 9'd0;
			flag <= 1;
		end
		else
			count <= count + 1'b1;
	end
	else
	begin
		count <= count + 1'b1;
		flag <= 0;
	end
end
		
//generate 'csb' pulse
always @ (posedge clk)
	if (count == tcsbh)
		csb <= 1'b1;
	else if (count == tcsbh + csb_length)
		csb <= 1'b0;
	else
		csb <= csb;	

//generate 'sck_gate' pulse		
always @ (posedge clk)
	if (count == tsckgateh)
		sck_gate <= 1'b1;
	else if (count == tsckgateh + sck_length)
		sck_gate <= 1'b0;
	else
		sck_gate <= sck_gate;
		
//generate 'rx_reset' pulse	
always @ (posedge clk)
	if (count == trx_reseth)
		rx_reset <= 1'b1;
	else if (count == trx_reseth + rx_reset_length)
		rx_reset <= 1'b0;
	else
		rx_reset <= rx_reset;

//generate 'cnvclk' pulse		
always @ (posedge clk)
	if (count == tcnvclkh)
		cnvclk <= 1'b1;
	else if (count == tcnvclkh + cnvclk_length)
		cnvclk <= 1'b0;
	else
		cnvclk <= cnvclk;	

endmodule